# -*- coding: utf-8 -*-
# Copyright 2014 Google Inc. All rights reserved.
#
# Modifications Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#

"""Performs requests to the ORS directions API."""

from openrouteservice import deprecation
from openrouteservice.optimization import optimization, Job, Vehicle

import warnings


def directions(client,
               coordinates,
               profile='driving-car',
               format_out=None,
               format='json',
               preference=None,
               units=None,
               language=None,
               geometry=None,
               geometry_simplify=None,
               instructions=None,
               instructions_format=None,
               roundabout_exits=None,
               attributes=None,
               maneuvers=None,
               radiuses=None,
               bearings=None,
               continue_straight=None,
               elevation=None,
               extra_info=None,
               suppress_warnings=None,
               optimized=None,
               optimize_waypoints=None,
               options=None,
               validate=True,
               dry_run=None):
    """Get directions between an origin point and a destination point.

    For more information, visit https://go.openrouteservice.org/documentation/.

    :param coordinates: The coordinates tuple the route should be calculated
        from. In order of visit.
    :type origin: list or tuple of coordinate lists or tuples

    :param profile: Specifies the mode of transport to use when calculating
        directions. One of ["driving-car", "driving-hgv", "foot-walking",
        "foot-hiking", "cycling-regular", "cycling-road","cycling-mountain",
        "cycling-electric",]. Default "driving-car".
    :type profile: string

    :param format: Specifies the response format. One of ['json', 'geojson', 'gpx']. Default "json".
        Geometry format for "json" is Google's encodedpolyline. The GPX schema the response is validated
        against can be found here:
        https://raw.githubusercontent.com/GIScience/openrouteservice-schema/master/gpx/v1/ors-gpx.xsd.
    :type format: str

    :param format_out: DEPRECATED.
    :type format: str

    :param preference: Specifies the routing preference. One of ["fastest, "shortest",
        "recommended"]. Default "fastest".
    :type preference: string

    :param units: Specifies the distance unit. One of ["m", "km", "mi"]. Default "m".
    :type units: string

    :param language: Language for routing instructions. One of ["en", "de", "cn",
        "es", "ru", "dk", "fr", "it", "nl", "br", "se", "tr", "gr"]. Default "en".
    :type language: string

    :param language: The language in which to return results.
    :type language: string

    :param geometry: Specifies whether geometry should be returned. Default True.
    :type geometry: boolean

    :param geometry_simplify: Specifies whether to simplify the geometry.
        Default False.
    :type geometry_simplify: boolean

    :param instructions: Specifies whether to return turn-by-turn instructions.
        Default True.
    :type instructions: boolean

    :param instructions_format: Specifies the the output format for instructions.
        One of ["text", "html"]. Default "text".
    :type instructions_format: string

    :param roundabout_exits: Provides bearings of the entrance and all passed
        roundabout exits. Adds the 'exit_bearings' array to the 'step' object
        in the response. Default False.
    :type roundabout_exits: boolean

    :param attributes: Returns route attributes on ["detourfactor", "percentage"].
        Must be a list of strings. Default None.
    :type attributes: list or tuple of strings

    :param maneuvers: Specifies whether the maneuver object is included into the step object or not. Default: false.
    :type maneuvers bool

    :param radiuses: A list of maximum distances (measured in
        meters) that limit the search of nearby road segments to every given waypoint.
        The values must be greater than 0, the value of -1 specifies no limit in
        the search. The number of radiuses must correspond to the number of waypoints.
        Default None.
    :type radiuses: list or tuple

    :param bearings: Specifies a list of pairs (bearings and
        deviations) to filter the segments of the road network a waypoint can
        snap to. For example bearings=[[45,10],[120,20]]. Each pair is a
        comma-separated list that can consist of one or two float values, where
        the first value is the bearing and the second one is the allowed deviation
        from the bearing. The bearing can take values between 0 and 360 clockwise
        from true north. If the deviation is not set, then the default value of
        100 degrees is used. The number of pairs must correspond to the number
        of waypoints. Setting optimized=false is mandatory for this feature to
        work for all profiles. The number of bearings corresponds to the length
        of waypoints-1 or waypoints. If the bearing information for the last waypoint
        is given, then this will control the sector from which the destination
        waypoint may be reached.
    :type bearings: list or tuple or lists or tuples

    :param continue_straight: Forces the route to keep going straight at waypoints
        restricting u-turns even if u-turns would be faster. This setting
        will work for all profiles except for driving-*. In this case you will
        have to set optimized=false for it to work. True or False. Default False.
    :type continue_straight: boolean

    :param elevation: Specifies whether to return elevation values for points.
        Default False.
    :type elevation: boolean

    :param extra_info: Returns additional information on ["steepness", "suitability",
        "surface", "waycategory", "waytype", "tollways", "traildifficulty", "roadaccessrestrictions"].
        Must be a list of strings. Default None.
    :type extra_info: list or tuple of strings

    :param suppress_warnings: Tells the system to not return any warning messages and corresponding extra_info.
        For false the extra information can still be explicitly requested by adding it with the extra_info parameter.
    :type suppress_warnings: bool

    :param optimized: If set False, forces to not use Contraction Hierarchies.
    :type optimized: bool

    :param options: Refer to https://go.openrouteservice.org/documentation for
        detailed documentation. Construct your own dict() following the example
        of the minified options object. Will be converted to json automatically.
    :type options: dict

    :param optimize_waypoints: If True, a `Vroom <https://github.com/VROOM-Project/vroom>`_ instance (ORS optimization
        endpoint) will optimize the `via` waypoints, i.e. all coordinates between the first and the last. It assumes
        the first coordinate to be the start location and the last coordinate to be the end location. Only requests with
        a minimum of 4 coordinates, no routing options and fastest weighting. Default False.
    :type optimize_waypoints: bool

    :param validate: Specifies whether parameters should be validated before sending the request. Default True.
    :type validate: bool
    
    :param dry_run: Print URL and parameters without sending the request.
    :param dry_run: boolean

    :raises ValueError: When parameter has wrong value.
    :raises TypeError: When parameter is of wrong type.

    :returns: sanitized set of parameters
    :rtype: call to Client.request()
    """

    # call optimization endpoint and get new order of waypoints
    if optimize_waypoints is not None and not dry_run:
        if len(coordinates) <= 3:
            warnings.warn("Less than 4 coordinates, nothing to optimize!", UserWarning)
        elif options:
            warnings.warn("Options are not compatible with optimization.", UserWarning)
        elif preference == 'shortest':
            warnings.warn("Shortest is not compatible with optimization.", UserWarning)
        else:
            coordinates = _optimize_waypoint_order(client, coordinates, profile)

    params = {"coordinates": coordinates}

    if format_out:
        deprecation.warning('format_out', 'format')

    format = format_out or format

    if preference:
        params["preference"] = preference

    if units:
        params["units"] = units

    if language:
        params["language"] = language

    if geometry is not None:
        params["geometry"] = geometry

    if geometry_simplify is not None:
        params["geometry_simplify"] = geometry_simplify

    if instructions is not None:
        params["instructions"] = instructions

    if instructions_format:
        params["instructions_format"] = instructions_format

    if roundabout_exits is not None:
        params["roundabout_exits"] = roundabout_exits

    if attributes:
        params["attributes"] = attributes

    if radiuses:
        params["radiuses"] = radiuses

    if maneuvers is not None:
        params['maneuvers'] = maneuvers

    if bearings:
        params["bearings"] = bearings

    if continue_straight is not None:
        params["continue_straight"] = continue_straight

    if elevation is not None:
        params["elevation"] = elevation

    if extra_info:
        params["extra_info"] = extra_info

    if suppress_warnings is not None:
        params['suppress_warnings'] = suppress_warnings

    if optimized is not None:
        if (bearings or continue_straight) and optimized in (True, 'true'):
            params["optimized"] = 'false'
            print("Set optimized='false' due to incompatible parameter settings.")
        else:
            params["optimized"] = optimized

    if options:
        params['options'] = options

    return client.request("/v2/directions/" + profile + '/' + format, {}, post_json=params, dry_run=dry_run)


def _optimize_waypoint_order(client, coordinates, profile):

    start = coordinates[0]
    end = coordinates[-1]
    veh = [Vehicle(
        id=0,
        profile=profile,
        start=start,
        end=end
    )]

    jobs = []
    for idx, coord in enumerate(coordinates[1:-1]):
        jobs.append(Job(
            id=idx,
            location=coord
        ))

    params = {
        'jobs': jobs,
        'vehicles': veh
    }

    optimization_res = optimization(client, **params)

    coordinates = []
    for step in optimization_res['routes'][0]['steps']:
        coordinates.append(step['location'])

    return coordinates



# -*- coding: utf-8 -*-

PARAM_POINT = [8.34234, 48.23424]
PARAM_LINE = [[8.688641, 49.420577], [8.680916, 49.415776]]
PARAM_POLY = [[[8.688641, 49.420577], [8.680916, 49.415776]]]

PARAM_INT_BIG = 500
PARAM_INT_SMALL = 50
PARAM_LIST_ONE = [PARAM_INT_SMALL, PARAM_INT_SMALL]
PARAM_LIST_TWO = [PARAM_LIST_ONE, PARAM_LIST_ONE]

PARAM_GEOJSON_POINT = {'type': 'Point', 'coordinates': PARAM_POINT}
PARAM_GEOJSON_LINE = {'type': 'LineString', 'coordinates': PARAM_LINE}
PARAM_GEOJSON_POLY = {'type': 'Polygon', 'coordinates': PARAM_POLY}

ENDPOINT_DICT = {
    'directions': {
        'coordinates': PARAM_LINE,
        'profile': 'driving-car',
        'preference': 'fastest',
        'format': 'geojson',
        'units': 'mi',
        'language': 'en',
        'geometry': 'true',
        'geometry_simplify': 'false',
        'maneuvers': True,
        'suppress_warnings': False,
        'instructions': 'false',
        'instructions_format': 'html',
        'roundabout_exits': 'true',
        'attributes': ['avgspeed'],
        'radiuses': PARAM_LIST_ONE,
        'bearings': PARAM_LIST_TWO,
        'elevation': 'true',
        'extra_info': ['roadaccessrestrictions'],
        'optimized': 'false',
        'continue_straight': True,
        'options': {'avoid_features': ['highways', 'tollways']}
    },
    'isochrones': {
        'locations': PARAM_LINE,
        'profile': 'cycling-regular',
        'range_type': 'distance',
        'range': [PARAM_INT_BIG],
        'units': 'm',
        'location_type': 'destination',
        'attributes': ['area', 'reachfactor'],
        'interval': [PARAM_INT_SMALL]
    },
    'distance_matrix': {
        'locations': PARAM_LINE,
        'sources': [1],
        'destinations': [0],
        'profile': 'driving-car',
        'metrics': ['duration', 'distance'],
        'resolve_locations': 'true',
        'units': 'mi',
        'optimized': 'false'
    },
    'elevation_point': {
        'format_in': 'geojson',
        'format_out': 'point',
        'geometry': PARAM_GEOJSON_POINT,
        'dataset': 'srtm'
    },
    'elevation_line': {
        'format_in': 'geojson',
        'format_out': 'polyline',
        'geometry': PARAM_GEOJSON_LINE,
        'dataset': 'srtm'
    },
    'pelias_search': {
        'text': 'Heidelberg',
        'focus_point': PARAM_POINT,
        'rect_min_x': PARAM_INT_BIG,
        'rect_min_y': PARAM_INT_BIG,
        'rect_max_x': PARAM_INT_BIG + 1,
        'rect_max_y': PARAM_INT_BIG + 1,
        'circle_point': PARAM_POINT,
        'circle_radius': PARAM_INT_SMALL,
        'sources': ['osm', 'wof', 'gn'],
        'layers': ['locality', 'county', 'region'],
        'country': 'de',
        'size': PARAM_INT_SMALL,
    },
    'pelias_autocomplete': {
        'text': 'Heidelberg',
        'focus_point': PARAM_POINT,
        'rect_min_x': PARAM_INT_BIG,
        'rect_min_y': PARAM_INT_BIG,
        'rect_max_x': PARAM_INT_BIG,
        'rect_max_y': PARAM_INT_BIG,
        'sources': ['osm', 'wof', 'gn'],
        'layers': ['locality', 'county', 'region'],
        'country': 'de',
    },
    'pelias_structured': {
        'address': 'Berliner Straße 45',
        'neighbourhood': 'Neuenheimer Feld',
        'borough': 'Heidelberg',
        'locality': 'Heidelberg',
        'county': 'Rhein-Neckar-Kreis',
        'region': 'Baden-Württemberg',
        'postalcode': 69120,
        'country': 'de',
    },
    'pelias_reverse': {
        'point': PARAM_POINT,
        'circle_radius': PARAM_INT_SMALL,
        'sources': ['osm', 'wof', 'gn'],
        'layers': ['locality', 'county', 'region'],
        'country': 'de',
        'size': PARAM_INT_SMALL,
    },
    'pois': {
        'request': 'pois',
        'geojson': PARAM_GEOJSON_POINT,
        'bbox': PARAM_LINE,
        'buffer': PARAM_INT_SMALL,
        'filter_category_ids': [PARAM_INT_SMALL],
        'filter_category_group_ids': [PARAM_INT_BIG],
        'filters_custom': {
            'name': 'Deli',
            'wheelchair': ['yes', 'limited'],
            'smoking': ['dedicated', 'separated'],
            'fee': ['yes', 'no']
        },
        'limit': PARAM_INT_SMALL,
        'sortby': 'distance',
    },
    'optimization': {
        "jobs": [
          {
            "id": 0,
            "location": PARAM_LINE[0],
            "location_index": 0,
            "service": PARAM_INT_BIG,
            "amount": [PARAM_INT_SMALL],
            "skills": PARAM_LIST_ONE,
            "time_windows": [PARAM_LIST_ONE]
          },
          {
            "id": 1,
            "location": PARAM_LINE[1],
            "location_index": 1,
            "service": PARAM_INT_BIG,
            "amount": [PARAM_INT_SMALL],
            "skills": PARAM_LIST_ONE,
            "time_windows": [PARAM_LIST_ONE]
          }
        ],
        "vehicles": [
          {
            "id": 0,
            "profile": "driving-car",
            "start": PARAM_LINE[0],
            "start_index": 0,
            "end_index": 0,
            "end": PARAM_LINE[0],
            "capacity": [
              PARAM_INT_SMALL
            ],
            "skills": PARAM_LIST_ONE,
            "time_window": PARAM_LIST_ONE
          },
          {
            "id": 1,
            "profile": "driving-car",
            "start": PARAM_LINE[1],
            "start_index": 1,
            "end_index": 1,
            "end": PARAM_LINE[1],
            "capacity": [
              PARAM_INT_SMALL
            ],
            "skills": PARAM_LIST_ONE,
            "time_window": PARAM_LIST_ONE
          },
        ],
        "options": {
            'g': False
        },
        "matrix": PARAM_LIST_TWO
    }
}


import warnings

from openrouteservice import deprecation

with warnings.catch_warnings(record=True) as w:
    # Cause all warnings to always be triggered.
    warnings.simplefilter("always")
    # Trigger a warning.
    deprecation.warning('foo', 'bar')
    # Verify some things
    assert len(w) == 1
    assert issubclass(w[-1].category, DeprecationWarning)
    assert "deprecated" in str(w[-1].message)



# -*- coding: utf-8 -*-
# Copyright 2014 Google Inc. All rights reserved.
#
# Modifications Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.


"""Tests for the Pelias geocoding module."""

import responses

import test as _test
from test.test_helper import ENDPOINT_DICT


class GeocodingPeliasTest(_test.TestCase):
    search = ENDPOINT_DICT['pelias_search']
    autocomplete = ENDPOINT_DICT['pelias_autocomplete']
    structured = ENDPOINT_DICT['pelias_structured']
    reverse = ENDPOINT_DICT['pelias_reverse']

    @responses.activate
    def test_pelias_search(self):
        responses.add(responses.GET,
                      'https://api.openrouteservice.org/geocode/search',
                      body='{"status":"OK","results":[]}',
                      status=200,
                      content_type='application/json')

        results = self.client.pelias_search(**self.search)

        self.assertEqual(1, len(responses.calls))
        self.assertURLEqual(
            'https://api.openrouteservice.org/geocode/search?boundary.circle.lat=48.23424&boundary.circle.lon=8.34234&boundary.circle.radius=50&boundary.country=de&boundary.rect.max_lat=501&boundary.rect.max_lon=501&boundary.rect.min_lat=500&boundary.rect.min_lon=500&focus.point.lat=48.23424&focus.point.lon=8.34234&layers=locality%2Ccounty%2Cregion&size=50&sources=osm%2Cwof%2Cgn&text=Heidelberg',
            responses.calls[0].request.url)

    @responses.activate
    def test_pelias_autocomplete(self):
        responses.add(responses.GET,
                      'https://api.openrouteservice.org/geocode/autocomplete',
                      body='{"status":"OK","results":[]}',
                      status=200,
                      content_type='application/json')

        results = self.client.pelias_autocomplete(**self.autocomplete)

        self.assertEqual(1, len(responses.calls))
        self.assertURLEqual(
            'https://api.openrouteservice.org/geocode/autocomplete?boundary.country=de&boundary.rect.max_lon%09=500&boundary.rect.min_lat%09=500&boundary.rect.min_lon%09=500&focus.point.lat=48.23424&focus.point.lon=8.34234&layers=locality%2Ccounty%2Cregion&sources=osm%2Cwof%2Cgn&text=Heidelberg',
            responses.calls[0].request.url)

    @responses.activate
    def test_pelias_structured(self):
        responses.add(responses.GET,
                      'https://api.openrouteservice.org/geocode/search/structured',
                      body='{"status":"OK","results":[]}',
                      status=200,
                      content_type='application/json')

        results = self.client.pelias_structured(**self.structured)

        self.assertEqual(1, len(responses.calls))
        self.assertURLEqual(
            'https://api.openrouteservice.org/geocode/search/structured?address=Berliner+Stra%C3%9Fe+45&borough=Heidelberg&country=de&county=Rhein-Neckar-Kreis&locality=Heidelberg&neighbourhood=Neuenheimer+Feld&postalcode=69120&region=Baden-W%C3%BCrttemberg',
            responses.calls[0].request.url)

    @responses.activate
    def test_pelias_reverse(self):
        responses.add(responses.GET,
                      'https://api.openrouteservice.org/geocode/reverse',
                      body='{"status":"OK","results":[]}',
                      status=200,
                      content_type='application/json')

        results = self.client.pelias_reverse(**self.reverse)

        self.assertEqual(1, len(responses.calls))
        self.assertURLEqual(
            'https://api.openrouteservice.org/geocode/reverse?boundary.circle.radius=50&boundary.country=de&layers=locality%2Ccounty%2Cregion&point.lat=48.23424&point.lon=8.34234&size=50&sources=osm%2Cwof%2Cgn',
            responses.calls[0].request.url)



# -*- coding: utf-8 -*-
# Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#

"""Tests for the distance matrix module."""
import responses
import test as _test
from test.test_helper import PARAM_POINT, ENDPOINT_DICT


class IsochronesTest(_test.TestCase):

    @responses.activate
    def test_isochrones(self):
        query = ENDPOINT_DICT['isochrones']

        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/isochrones/{}/geojson'.format(query['profile']),
                      json=query,
                      status=200,
                      content_type='application/json')

        resp = self.client.isochrones(**query)

        self.assertEqual(1, len(responses.calls))
        self.assertEqual(resp, query)



# -*- coding: utf-8 -*-
# Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#

"""Performs requests to the ORS optimization API."""


def optimization(client,
                 jobs,
                 vehicles,
                 matrix=None,
                 geometry=None,
                 dry_run=None):
    """Optimize a fleet of vehicles on a number of jobs.

    For more information, visit https://github.com/VROOM-Project/vroom/blob/master/docs/API.md.

    Example:

        >>> from openrouteservice import Client, optimization
        >>> coordinates = [[8.688641, 49.420577], [8.680916, 49.415776]]
        >>> jobs, vehicles = list(), list()
        >>> for idx, coord in enumerate(coordinates):
                jobs.append(optimization.Job(id=idx, location=coord))
                vehicles.append(optimization.Vehicle(id=idx, location=coord))
        >>> api = Client(key='somekey')
        >>> result = api.optimization(jobs=jobs, vehicles=vehicles)

    :param jobs: The Job objects to fulfill.
    :type jobs: list of :class:`openrouteservice.optimization.Job`

    :param vehicles: The vehicles to fulfill the :class:`openrouteservice.optimization.Job`'s.
    :type vehicles: list of :class:`openrouteservice.optimization.Vehicle`

    :param matrix: Specify a custom cost matrix. If not specified, it will be calculated with
        the :meth:`openrouteservice.matrix.matrix` endpoint.
    :type matrix: list of lists of int

    :param geometry: If the geometry of the resulting routes should be calculated. Default False.
    :type geometry: bool

    :param dry_run: Print URL and parameters without sending the request.
    :param dry_run: boolean

    :returns: Response of optimization endpoint.
    :rtype: dict
    """

    assert all([isinstance(x, Job) for x in jobs])
    assert all([isinstance(x, Vehicle) for x in vehicles])

    params = {"jobs": [job.__dict__ for job in jobs],
              "vehicles": [vehicle.__dict__ for vehicle in vehicles]}

    if geometry is not None:
        params.update({"options": {"g": geometry}})

    if matrix:
        params['matrix'] = matrix

    return client.request("/optimization", {}, post_json=params, dry_run=dry_run)


class Job(object):
    """
    Class to create a Job object for optimization endpoint.

    Full documentation at https://github.com/VROOM-Project/vroom/blob/master/docs/API.md#jobs.
    """
    def __init__(self,
                 id,
                 location=None,
                 location_index=None,
                 service=None,
                 amount=None,
                 skills=None,
                 time_windows=None
                 ):
        """
        Create a job object for the optimization endpoint.

        :param id: Integer used as unique identifier.
        :type id: int

        :param location: Location of the job, as [lon, lat]. Optional if custom matrix is provided.
        :type location: tuple of float or list of float

        :param location_index: Index of relevant row and column in custom matrix. Mandatory if custom
            matrix is provided. Irrelevant when no custom matrix is provided.
        :type location_index: int

        :param service: Optional job service duration in seconds
        :type service: int

        :param amount: An array of integers describing multidimensional quantities.
        :type amount: list of int or tuple of int

        :param skills: An array of integers defining mandatory skills for this job.
        :type skills: list of int or tuple of int

        :param time_windows: An array of time_window objects describing valid slots for job service start.
        :type time_windows: list of lists of int
        """

        self.id = id

        if location is not None:
            self.location = location

        if location_index is not None:
            self.location_index = location_index

        if service is not None:
            self.service = service

        if amount is not None:
            self.amount = amount

        if skills is not None:
            self.skills = skills

        if time_windows is not None:
            self.time_windows = time_windows


class Vehicle(object):
    """
    Class to create a Vehicle object for optimization endpoint.

    Full documentation at https://github.com/VROOM-Project/vroom/blob/master/docs/API.md#vehicles.
    """

    def __init__(self,
                 id,
                 profile='driving-car',
                 start=None,
                 start_index=None,
                 end=None,
                 end_index=None,
                 capacity=None,
                 skills=None,
                 time_window=None):
        """
        Create a Vehicle object for the optimization endpoint.

        :param id: Integer used as unique identifier.
        :param id: int

        :param profile: Specifies the mode of transport to use when calculating
            directions. One of ["driving-car", "driving-hgv", "foot-walking",
            "foot-hiking", "cycling-regular", "cycling-road","cycling-mountain",
            "cycling-electric",]. Default "driving-car".
        :type profile: str

        :param start: Coordinate for the vehicle to start from. If not specified, the start
            location will be the first visited job, determined by the optimization engine.
        :type start: list of float or tuple of float

        :param start_index: Index of relevant row and column in custom matrix. Irrelevant
            if no custom matrix is provided.
        :type start_index: int

        :param end: Coordinate for the vehicle to end at. If not specified, the end
            location will be the last visited job, determined by the optimization engine.
        :type end: list of float or tuple of float

        :param end_index: Index of relevant row and column in custom matrix. Irrelevant
            if no custom matrix is provided.
        :type end_index: int

        :param capacity: An array of integers describing multidimensional quantities.
        :type capacity: list of int or tuple of int

        :param skills: An array of integers defining skills for this vehicle.
        :param skills: list of int or tuple of int

        :param time_window: A time_window object describing working hours for this vehicle.
        :param time_window: list of int or tuple of int
        """

        self.id = id
        self.profile = profile

        if start is not None:
            self.start = start

        if start_index is not None:
            self.start_index = start_index

        if end is not None:
            self.end = end

        if end_index is not None:
            self.end_index = end_index

        if capacity is not None:
            self.capacity = capacity

        if skills is not None:
            self.skills = skills

        if time_window is not None:
            self.time_window = time_window



# -*- coding: utf-8 -*-
# Copyright 2014 Google Inc. All rights reserved.
#
# Modifications Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#

"""Performs requests to the ORS Matrix API."""


def distance_matrix(client, locations,
                    profile='driving-car',
                    sources=None,
                    destinations=None,
                    metrics=None,
                    resolve_locations=None,
                    units=None,
                    optimized=None,
                    validate=True,
                    dry_run=None):
    """ Gets travel distance and time for a matrix of origins and destinations.

    :param locations: One or more pairs of lng/lat values.
    :type locations: a single location, or a list of locations, where a
        location is a list or tuple of lng,lat values

    :param profile: Specifies the mode of transport to use when calculating
        directions. One of ["driving-car", "driving-hgv", "foot-walking",
        "foot-hiking", "cycling-regular", "cycling-road",
        "cycling-safe", "cycling-mountain", "cycling-tour", 
        "cycling-electric",]. Default "driving-car".
    :type profile: string

    :param sources: A list of indices that refer to the list of locations
        (starting with 0). If not passed, all indices are considered.
    :type sources: list or tuple

    :param destinations: A list of indices that refer to the list of locations
        (starting with 0). If not passed, all indices are considered.
    :type destinations: list or tuple

    :param metrics: Specifies a list of returned metrics. One or more of ["distance",
        "duration"]. Default ['duration'].
    :type metrics: list of strings

    :param resolve_locations: Specifies whether given locations are resolved or
        not. If set 'true', every element in destinations and sources will 
        contain the name element that identifies the name of the closest street.
        Default False.
    :type resolve_locations: boolean

    :param units: Specifies the unit system to use when displaying results.
        One of ["m", "km", "m"]. Default "m".
    :type units: string

    :param optimized: Specifies whether Dijkstra algorithm ('false') or any 
        available technique to speed up shortest-path routing ('true') is used. 
        For normal Dijkstra the number of visited nodes is limited to 100000.
        Default True
    :type optimized: boolean

    :param validate: Specifies whether parameters should be validated before sending the request. Default True.
    :type validate: bool
    
    :param dry_run: Print URL and parameters without sending the request.
    :param dry_run: boolean
    
    :raises ValueError: When profile parameter has wrong value.
    
    :rtype: call to Client.request()
    """

    params = {
        "locations": locations,
    }

    if sources:
        params['sources'] = sources

    if destinations:
        params['destinations'] = destinations

    if profile:
        params["profile"] = profile

    if sources:
        params["sources"] = sources

    if destinations:
        params["destinations"] = destinations

    if metrics:
        params["metrics"] = metrics

    if resolve_locations is not None:
        params["resolve_locations"] = resolve_locations

    if units:
        params["units"] = units

    if optimized is not None:
        params["optimized"] = optimized

    return client.request("/v2/matrix/" + profile + '/json', {}, post_json=params, dry_run=dry_run)



# -*- coding: utf-8 -*-

# Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

"""Tests for the distance matrix module."""
import responses
import test as _test

from test.test_helper import ENDPOINT_DICT


class ElevationTest(_test.TestCase):
    valid_query = ENDPOINT_DICT['elevation_line']

    @responses.activate
    def test_elevation_line(self):
        responses.add(responses.POST,
                      'https://api.openrouteservice.org/elevation/line',
                      json=self.valid_query,
                      status=200,
                      content_type='application/json')

        resp = self.client.elevation_line(**self.valid_query)

        self.assertEqual(len(responses.calls), 1)
        self.assertEqual(resp, self.valid_query)

    @responses.activate
    def test_elevation_point(self):
        query = ENDPOINT_DICT['elevation_point']
        responses.add(responses.POST,
                      'https://api.openrouteservice.org/elevation/point',
                      json=self.valid_query,
                      status=200,
                      content_type='application/json')

        resp = self.client.elevation_point(**query)

        self.assertEqual(len(responses.calls), 1)
        self.assertEqual(resp, self.valid_query)



# -*- coding: utf-8 -*-
# Copyright 2014 Google Inc. All rights reserved.
#
# Modifications Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#

"""Tests for the distance matrix module."""
import responses
import test as _test
from copy import deepcopy
import json

from test.test_helper import *
from openrouteservice.optimization import Job, Vehicle


class OptimizationTest(_test.TestCase):

    def _get_params(self):
        jobs, vehicles = list(), list()

        for idx, coord in enumerate(PARAM_LINE):
            jobs.append(Job(idx, location=coord,
                             service=PARAM_INT_BIG,
                             location_index=idx,
                             amount=[PARAM_INT_SMALL],
                             skills=PARAM_LIST_ONE,
                             time_windows=[PARAM_LIST_ONE]
                             ))

            vehicles.append(Vehicle(idx, profile='driving-car',
                                     start=coord,
                                     start_index=idx,
                                     end=coord,
                                     end_index=idx,
                                     capacity=[PARAM_INT_SMALL],
                                     skills=PARAM_LIST_ONE,
                                     time_window=PARAM_LIST_ONE))
        return jobs, vehicles

    def test_jobs_vehicles_classes(self):

        jobs, vehicles = self._get_params()

        self.assertEqual(ENDPOINT_DICT['optimization']['jobs'], [j.__dict__ for j in jobs])
        self.assertEqual(ENDPOINT_DICT['optimization']['vehicles'], [v.__dict__ for v in vehicles])

    @responses.activate
    def test_full_optimization(self):
        query = deepcopy(ENDPOINT_DICT['optimization'])

        jobs, vehicles = self._get_params()

        responses.add(responses.POST,
                      'https://api.openrouteservice.org/optimization',
                      json={},
                      status=200,
                      content_type='application/json')

        self.client.optimization(jobs, vehicles, geometry=False, matrix=PARAM_LIST_TWO)

        self.assertEqual(query, json.loads(responses.calls[0].request.body.decode('utf-8')))



# -*- coding: utf-8 -*-
# Copyright 2014 Google Inc. All rights reserved.
#
# Modifications Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#


"""Tests for client module."""

import responses
import time

import openrouteservice
from test.test_helper import *
import test as _test


class ClientTest(_test.TestCase):

    def test_no_api_key(self):
        with self.assertRaises(ValueError):
            client = openrouteservice.Client()
            client.directions(PARAM_LINE)

    def test_invalid_api_key(self):
        with self.assertRaises(openrouteservice.exceptions.ApiError):
            client = openrouteservice.Client(key="Invalid key.")
            client.directions(PARAM_LINE)

    def test_urlencode(self):
        encoded_params = openrouteservice.client._urlencode_params([("address", "=Sydney ~")])
        self.assertEqual("address=%3DSydney+~", encoded_params)

    @responses.activate
    def test_raise_empty_body(self):
        valid_query = ENDPOINT_DICT['directions']
        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/non-existent',
                      status=404,
                      content_type='application/json')

        with self.assertRaises(openrouteservice.exceptions.HTTPError):
            client = openrouteservice.Client(key=self.key)
            client.request('/v2/non-existent', get_params={}, post_json=valid_query)


    @responses.activate
    def test_raise_over_query_limit(self):
        valid_query = ENDPOINT_DICT['directions']
        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/directions/{}/geojson'.format(valid_query['profile']),
                      json=valid_query,
                      status=429,
                      content_type='application/json')

        with self.assertRaises(openrouteservice.exceptions._OverQueryLimit):
            client = openrouteservice.Client(key=self.key, retry_over_query_limit=False)
            client.directions(**valid_query)

        with self.assertRaises(openrouteservice.exceptions.Timeout):
            client = openrouteservice.Client(key=self.key, retry_over_query_limit=True, retry_timeout=3)
            client.directions(**valid_query)

    @responses.activate
    def test_raise_timeout_retriable_requests(self):
        # Mock query gives 503 as HTTP status, code should try a few times to 
        # request the same and then fail on Timout() error.
        retry_timeout = 3
        valid_query = ENDPOINT_DICT['directions']
        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/directions/{}/geojson'.format(valid_query['profile']),
                      json=valid_query,
                      status=503,
                      content_type='application/json')

        client = openrouteservice.Client(key=self.key,
                                         retry_timeout=retry_timeout)

        start = time.time()
        with self.assertRaises(openrouteservice.exceptions.Timeout):
            client.directions(**valid_query)
        end = time.time()
        self.assertTrue(retry_timeout < end - start < 2 * retry_timeout)

    @responses.activate
    def test_host_override_with_parameters(self):
        # Test if it's possible to override host for individual hosting.
        responses.add(responses.GET,
                      "https://foo.com/bar",
                      body='{"status":"OK","results":[]}',
                      status=200,
                      content_type="application/json")

        client = openrouteservice.Client(base_url="https://foo.com")
        client.request("/bar", {'bunny': 'pretty', 'fox': 'prettier'})

        self.assertURLEqual("https://foo.com/bar?bunny=pretty&fox=prettier",
                            responses.calls[0].request.url)
        self.assertEqual(1, len(responses.calls))

    @responses.activate
    def test_dry_run(self):
        # Test that nothing is requested when dry_run is 'true'

        responses.add(responses.GET,
                      'https://api.openrouteservice.org/directions',
                      body='{"status":"OK","results":[]}',
                      status=200,
                      content_type='application/json')

        req = self.client.request(get_params={'format_out': 'geojson'},
                                  url='directions/',
                                  dry_run='true')

        self.assertEqual(0, len(responses.calls))

    @responses.activate
    def test_key_in_header(self):
        # Test that API key is being put in the Authorization header
        query = ENDPOINT_DICT['directions']

        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/directions/{}/geojson'.format(query['profile']),
                      json=ENDPOINT_DICT['directions'],
                      status=200,
                      content_type='application/json')

        resp = self.client.directions(**query)

        self.assertDictContainsSubset({'Authorization': self.key}, responses.calls[0].request.headers)



# -*- coding: utf-8 -*-
import sys
import os.path

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.version_info <= (2, 6):
    error = 'Requires Python Version 2.7 or above... exiting.'
    print(sys.stderr, error)
    sys.exit(1)

here = os.path.abspath(os.path.dirname(__file__))


def readme():
    with open(os.path.join(here, 'README.rst')) as f:
        return f.read()


setup(
    name='openrouteservice',
    version='2.2.3',
    description='Python client for requests to openrouteservice API services',
    long_description=readme(),
    long_description_content_type='text/x-rst',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    keywords='routing accessibility router OSM ORS openrouteservice openstreetmap isochrone POI elevation DEM',
    url='https://github.com/GIScience/openrouteservice-py',
    author='Nils Nolde',
    author_email='nils.nolde@gmail.com',
    license='Apache-2.0',
    packages=['openrouteservice'],
    install_requires=[
        'requests>=2.0'],
    include_package_data=True,
    test_suite='nose.collector',
    tests_require=['nose>1.0',
                   'requests>=2.0',
                   'responses>=0.10',
                   'coveralls>=1.7.0',
                   'coverage>=4.5.0'],
    zip_safe=False
)



from openrouteservice.exceptions import ValidationError, ApiError, \
    HTTPError, Timeout, _RetriableRequest, _OverQueryLimit

import test as _test

from pprint import pprint


class ExceptionTest(_test.TestCase):

    def test_ValidationError(self):
        exception = ValidationError('hamspam')

        pprint(exception.__dict__)
        self.assertIsInstance(exception, Exception)

    def test_ApIError(self):
        exception = ApiError(500, 'hamspam')

        pprint(exception.__dict__)

        self.assertEqual(exception.status, 500)
        self.assertEqual(exception.message, 'hamspam')

        self.assertEqual(str(exception), '500 (hamspam)')

        exception = ApiError(500)

        self.assertEqual(str(exception), '500')

    def test_HTTPError(self):
        exception = HTTPError(500)

        self.assertEqual(exception.status_code, 500)

        self.assertEqual(str(exception), 'HTTP Error: 500')

    def test_Timeout(self):
        exception = Timeout()

        self.assertIsInstance(exception, Exception)

    def test_RetriableRequest(self):
        exception = _RetriableRequest()

        self.assertIsInstance(exception, Exception)

    def test_OverQueryLimit(self):
        exception = _OverQueryLimit(500, 'hamspam')

        self.assertIsInstance(exception, Exception)
        self.assertIsInstance(exception, ApiError)
        self.assertIsInstance(exception, _RetriableRequest)

        self.assertEqual(str(exception), '500 (hamspam)')



# -*- coding: utf-8 -*-
# Copyright 2014 Google Inc. All rights reserved.
#
# Modifications Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#

"""Tests for the directions module."""

import responses
import warnings

import test as _test
from copy import deepcopy

import openrouteservice
from test.test_helper import ENDPOINT_DICT


class DirectionsTest(_test.TestCase):
    valid_query = ENDPOINT_DICT['directions']

    @responses.activate
    def test_directions(self):
        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/directions/{}/geojson'.format(self.valid_query['profile']),
                      json=self.valid_query,
                      status=200,
                      content_type='application/json')

        resp = self.client.directions(**self.valid_query)

        self.assertEqual(resp, self.valid_query)
        self.assertIn('sample_key', responses.calls[0].request.headers.values())

    def test_format_out_deprecation(self):
        bad_query = deepcopy(self.valid_query)
        bad_query['format_out'] = "json"
        bad_query['dry_run'] = True

        with warnings.catch_warnings(record=True) as w:
            # Cause all warnings to always be triggered.
            warnings.simplefilter("always")
            # Trigger a warning.
            _ = self.client.directions(**bad_query)

            assert len(w) == 1
            assert issubclass(w[-1].category, DeprecationWarning)
            assert "deprecated" in str(w[-1].message)

    def test_optimized_waypoints(self):
        query = deepcopy(self.valid_query)
        query['coordinates'] = [[8.688641, 49.420577], [8.680916, 49.415776],[8.688641, 49.420577], [8.680916, 49.415776]]
        query['optimize_waypoints'] = True

        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/directions/{}/geojson'.format(query['profile']),
                      json=query,
                      status=200,
                      content_type='application/json')

        # Too exhausting to really test this
        with self.assertRaises(openrouteservice.exceptions.ApiError):
            resp = self.client.directions(**query)

    @responses.activate
    def test_optimize_warnings(self):
        query = deepcopy(self.valid_query)
        query['optimize_waypoints'] = True

        # Test Coordinates

        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/directions/{}/geojson'.format(query['profile']),
                      json=query,
                      status=200,
                      content_type='application/json')

        with warnings.catch_warnings(record=True) as w:
            # Cause all warnings to always be triggered.
            warnings.simplefilter("always")

            resp = self.client.directions(**query)

            assert len(w) == 1
            assert issubclass(w[-1].category, UserWarning)
            assert "4 coordinates" in str(w[-1].message)

        # Test Options
        
        query['coordinates'] = [[8.688641, 49.420577], [8.680916, 49.415776],[8.688641, 49.420577], [8.680916, 49.415776]]

        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/directions/{}/geojson'.format(query['profile']),
                      json=query,
                      status=200,
                      content_type='application/json')

        with warnings.catch_warnings(record=True) as w:
            # Cause all warnings to always be triggered.
            warnings.simplefilter("always")

            resp = self.client.directions(**query)

            assert len(w) == 1
            assert issubclass(w[-1].category, UserWarning)
            assert "Options" in str(w[-1].message)

        # Test Preference

        query['options'] = None
        query['preference'] = 'shortest'

        responses.add(responses.POST,
                      'https://api.openrouteservice.org/v2/directions/{}/geojson'.format(query['profile']),
                      json=query,
                      status=200,
                      content_type='application/json')

        with warnings.catch_warnings(record=True) as w:
            # Cause all warnings to always be triggered.
            warnings.simplefilter("always")

            resp = self.client.directions(**query)

            assert len(w) == 1
            assert issubclass(w[-1].category, UserWarning)
            assert "Shortest" in str(w[-1].message)
